/*
    Copyright 2006 Changbeom Ahn

    This file is part of Cmg::Mines.

    Cmg::Mines is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Cmg::Mines is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cmg::Mines; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    $Id$
*/


#include <wx/wxprec.h>

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif // WX_PRECOMP

#include "mines.hpp"


IMPLEMENT_APP(MinesApp)


bool
MinesApp::OnInit()
{
    AppFrame *frame = new AppFrame();

    frame->Show(TRUE);

    return TRUE;
}


AppFrame::AppFrame() : wxFrame(0, -1, wxT("Mines"))
{
    new MinesControl(this);
}


BEGIN_EVENT_TABLE(MinesControl, wxWindow)
    EVT_PAINT(MinesControl::onPaint)
END_EVENT_TABLE()


MinesControl::MinesControl(wxWindow* parent) :
    wxWindow(parent, -1), field_(NUM_ROWS, NUM_COLS, NUM_MINES)
{
}


void
MinesControl::onPaint(wxPaintEvent& /* event */)
{
    wxPaintDC dc(this);

    field_.paint(dc);
}


Field::Field(size_t numRows, size_t numCols, size_t numMines) :
    numRows_(numRows), numCols_(numCols), numMines_(numMines)
{
    for (size_t row = 0; row < numRows_; ++row) {
        std::vector<Cell> v;
        for (size_t col = 0; col < numCols_; ++col) {
            bool hasMine = false;
            if (numMines > 0) {
                hasMine = true;
                --numMines;
            }
            v.push_back(Cell(row, col, hasMine));
        }
        cells_.push_back(v);
    }
}


void
Field::paint(wxDC& dc) const
{
    for (size_t row = 0; row < numRows_; ++row) {
        for (size_t col = 0; col < numCols_; ++col) {
            cells_[row][col].paint(dc);
        }
    }
}


void
Cell::paint(wxDC& dc) const
{
    dc.DrawRectangle(row_ * CELL_SIZE, col_ * CELL_SIZE, CELL_SIZE, CELL_SIZE);

    if (hasMine_) {
        dc.DrawCircle(row_ * CELL_SIZE + CELL_SIZE / 2,
            col_ * CELL_SIZE + CELL_SIZE / 2, CELL_SIZE / 4);
    }
}
