/*
    Copyright 2006 Changbeom Ahn

    This file is part of Cmg::Mines.

    Cmg::Mines is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Cmg::Mines is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cmg::Mines; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    $Id$
*/


#ifndef MINES_HPP
#define MINES_HPP


#include <vector>


class Cell {
public:
    Cell(size_t row, size_t col, bool hasMine) :
        row_(row), col_(col), hasMine_(hasMine) {}

    void setMine(bool val) { hasMine_ = val; }
    bool hasMine() const { return hasMine_; }

    void paint(wxDC&) const;

private:
    size_t row_;
    size_t col_;
    bool hasMine_;

    static const size_t CELL_SIZE = 16;
}; // class Cell


class Field {
public:
    Field(size_t, size_t, size_t);

    void paint(wxDC&) const;

private:
    std::vector<std::vector<Cell> > cells_;
    size_t numRows_;
    size_t numCols_;
    size_t numMines_;

    Field(const Field&);
    Field& operator=(const Field&);
}; // class Field


class MinesApp : public wxApp {
public:
    virtual bool OnInit(void);
}; // class MinesApp


DECLARE_APP(MinesApp)


class AppFrame : public wxFrame {
public:
    AppFrame();
}; // class AppFrame


class MinesControl : public wxWindow {
public:
    MinesControl(wxWindow*);

private:
    void onPaint(wxPaintEvent&);

    Field field_;

    static const size_t NUM_ROWS = 8;
    static const size_t NUM_COLS = 8;
    static const size_t NUM_MINES = 10;

    DECLARE_EVENT_TABLE();
}; // class MinesControl


#endif // MINES_HPP
